#! /usr/bin/env python3
#
# extension d'origine: https://github.com/prometheus42/libreoffice-ical-importer
# modifié par @Rafbor afin de fonctionner sous forme de macro.
#
from arrow.arrow import Arrow
from datetime import timedelta
from ics import Calendar
from ics.grammar.parse import ParseError

import os
import uno
import msgbox
import unohelper
from com.sun.star.util import DateTime
from com.sun.star.util import Time
from com.sun.star.lang import Locale
from com.sun.star.uno import RuntimeException
from com.sun.star.task import XJobExecutor

# appel depuis événement clic-droit sur une feuille, un paramètre est envoyé par Calc
def fill_table_cd(param):
    fill_table()

# lancement direct de la macro
def fill_table():
    """Fills the first worksheet with data from an iCalendar file.
    
    All events from a given iCalendar file are imported. Every column
    represents a attribute from the file and has a corrsponding header. The
    cells will be configured with a number format depending on the type of data
    of that column (datetime, timedelta, string, etc.)."""
    
    ctx = uno.getComponentContext()
    smgr = ctx.getServiceManager()
    DESK = 'com.sun.star.frame.Desktop'
    desktop = smgr.createInstanceWithContext(DESK , ctx)
    doc = desktop.getCurrentComponent()
    # on récupère dans le 4ème onglet en B1, le chemin du fichier ical à importer
    sheet = doc.Sheets[3]
    cell = sheet.getCellByPosition(1,0)
    # si pas renseigné, on prend calendar.ics par défaut
    if cell.String == "":
    	filepath = ".local/share/evolution/calendar/system/calendar.ics"
    else:
    	filepath = cell.String
    filename = os.path.expanduser('~') + os.sep + filepath
    # on se place sur 1er onglet
    sheet = doc.Sheets[0]
    # write attributes into table header
    attr = ['name', 'begin', 'end', 'duration', 'uid', 'description', 'created',
            'last_modified', 'location', 'url', 'transparent', 'alarms',
            'attendees', 'categories', 'status', 'organizer', 'classification']
    for column, name in enumerate(attr):
        cell = sheet.getCellByPosition(column, 0)
        cell.String = name
    # iterate over all events and add data to table
    for row, e in enumerate(read_ical_file(filename)):
        for column, name in enumerate(attr):
            cell = sheet.getCellByPosition(column, row+1)
            fill_cell_with_data(doc, getattr(e, name), cell)
    # mark all columns and set them to optimal width
    selection = sheet.getCellRangeByPosition(0, 1, len(attr)-1, 1)
    for c in selection.Columns:
        c.OptimalWidth = True
    # terminé
    show_message_box(ctx, 'Import', 'Importation terminée')

def read_ical_file(filename):
    """Reads an iCalendar file and returns all events from that file."""
    with open(filename, 'r') as calendar_file:
        c = Calendar(calendar_file.read())
    return c.events

def fill_cell_with_data(doc, data, cell):
    """Fills a single cell with given data.
    
    Before the data is stored in the cell of an worksheet, the type of the data
    is checked. All necessary conversion and type casts are done before the
    value of the cell is set."""
    NumberFormat = doc.getNumberFormats()
    LocalSettings = Locale()
    LocalSettings.Language = 'fr'
    if not data:
        cell.String = ''
    elif type(data) == str:
        cell.String =  data
    elif type(data) == Arrow:
        # write datetime into cell
        cell.String = data.format('DD.MM.YYYY HH:mm:ss')
        # set number format for cell
        FormatString = 'TT.MM.JJJJ HH:MM:SS'
        NumberFormatID = NumberFormat.queryKey(FormatString, LocalSettings, True)
        # check whether a value was found for the requested format (-1 == no value)
        if NumberFormatID == -1:  
            NumberFormatID = NumberFormat.addNew(FormatString, LocalSettings)
        cell.NumberFormat = NumberFormatID
    elif type(data) == timedelta:
        # write datetime into cell
        cell.String = str(data)
        # set number format for cell
        FormatString = 'HH:MM:SS'
        NumberFormatID = NumberFormat.queryKey(FormatString, LocalSettings, True)
        # check whether a value was found for the requested format (-1 == no value)
        if NumberFormatID == -1:  
            NumberFormatID = NumberFormat.addNew(FormatString, LocalSettings)
        cell.NumberFormat = NumberFormatID
    elif type(data) == set:
        cell.String = ', '.join([x for x in data])
    else:
        print('Unsupported type: {}'.format(type(data)))
        # return data without conversion to trigger an exception
        return data
        
def show_message_box(ctx, title, message):
    """Shows a message box with a single button to acknowledge the information.
    
    Source: https://docs.libreoffice.org/scripting/html/msgbox_8py_source.html
    """
    myBox = msgbox.MsgBox(ctx)
    myBox.addButton('OK')
    myBox.renderFromBoxSize(150)
    myBox.numberOflines = 2
    myBox.show(message, 0, title)

